@@ -1085,9 +1085,8 @@ static FILE * _open_stat_file(string stat_file)
     stat_fh = fopen(stat_file.c_str(), "w");
     if (!stat_fh)
     {
-        fprintf(stderr, "Unable to open objstat output file: %s\n"
+        end(1, false, "Unable to open objstat output file: %s\n"
                 "Error: %s", stat_file.c_str(), strerror(errno));
-        end(1);
     }
     return stat_fh;
 }
@@ -1129,8 +1128,8 @@ static void _write_stat_info()
     string all_desc;
     if (num_branches > 1)
     {
-        if (SysEnv.map_gen_range.get())
-            all_desc = SysEnv.map_gen_range.get()->describe();
+        if (SysEnv.map_gen_range)
+            all_desc = SysEnv.map_gen_range->describe();
         else
             all_desc = "All Levels";
         all_desc = "Levels included in AllLevels: " + all_desc + "\n";
@@ -1205,7 +1204,7 @@ void objstat_generate_stats()
         for (int dep = 1; dep <= brdepth[br]; ++dep)
         {
             const level_id lid(br, dep);
-            if (SysEnv.map_gen_range.get()
+            if (SysEnv.map_gen_range
                 && !SysEnv.map_gen_range->is_usable_in(lid))
             {
                 continue;
