@@ -46,8 +46,10 @@ bool is_dumpable_artefact(const item_def &item);
 string get_item_description(const item_def &item, bool verbose,
                             bool dump = false, bool lookup = false);
 
-void describe_feature_wide(const coord_def& pos, bool show_quote = false);
-void get_feature_desc(const coord_def &gc, describe_info &inf);
+void describe_feature_wide(const coord_def& pos);
+void describe_feature_type(dungeon_feature_type feat);
+string get_cloud_desc(cloud_type cloud, bool include_title = true);
+void get_feature_desc(const coord_def &gc, describe_info &inf, bool include_extra = true);
 
 bool describe_item(item_def &item, function<void (string&)> fixup_desc = nullptr);
 void get_item_desc(const item_def &item, describe_info &inf);
@@ -59,13 +61,15 @@ int describe_monsters(const monster_info &mi, bool force_seen = false,
 
 void get_monster_db_desc(const monster_info &mi, describe_info &inf,
                          bool &has_stat_desc, bool force_seen = false);
+branch_type serpent_of_hell_branch(monster_type m);
 string serpent_of_hell_flavour(monster_type m);
 
 string player_spell_desc(spell_type spell);
 void get_spell_desc(const spell_type spell, describe_info &inf);
 void describe_spell(spell_type spelled,
                     const monster_info *mon_owner = nullptr,
-                    const item_def* item = nullptr);
+                    const item_def* item = nullptr,
+                    bool show_booklist = false);
 
 void describe_ability(ability_type ability);
 
@@ -80,8 +84,9 @@ int hex_chance(const spell_type spell, const int hd);
 
 string get_command_description(const command_type cmd, bool terse);
 
-void print_description(const string &desc);
-void print_description(const describe_info &inf);
+int show_description(const string &body, const tile_def *tile = nullptr);
+int show_description(const describe_info &inf, const tile_def *tile = nullptr);
+string process_description(const describe_info &inf, bool include_title = true);
 
 const char* get_size_adj(const size_type size, bool ignore_medium = false);
 
@@ -96,29 +101,3 @@ int str_to_trap(const string &s);
 int count_desc_lines(const string& _desc, const int width);
 
 string extra_cloud_info(cloud_type cloud_type);
-
-
-class alt_desc_proc
-{
-public:
-    alt_desc_proc(int _w, int _h) { w = _w; h = _h; }
-
-    int width() { return w; }
-    int height() { return h; }
-
-    void nextline();
-    void print(const string &str);
-    static int count_newlines(const string &str);
-
-    // Remove trailing newlines.
-    static void trim(string &str);
-    // rfind consecutive newlines and truncate.
-    static bool chop(string &str);
-
-    void get_string(string &str);
-
-protected:
-    int w;
-    int h;
-    ostringstream ostr;
-};
