@@ -89,7 +89,7 @@ void save_game(bool leave_game, const char *bye = nullptr);
 // Save game without exiting (used when changing levels).
 void save_game_state();
 
-bool get_save_version(reader &file, int &major, int &minor);
+save_version get_save_version(reader &file);
 
 bool save_exists(const string& filename);
 bool restore_game(const string& filename);
@@ -113,6 +113,9 @@ void save_ghosts(const vector<ghost_demon> &ghosts, bool force = false,
                                                     bool use_store = true);
 bool load_ghosts(int max_ghosts, bool creating_level);
 bool define_ghost_from_bones(monster& mons);
+vector<ghost_demon> load_bones_file(string ghost_filename, bool backup=false);
+void write_ghost_version(writer &outf);
+save_version read_ghost_header(reader &inf);
 
 FILE *lk_open(const char *mode, const string &file);
 FILE *lk_open_exclusive(const string &file);
