@@ -465,13 +465,16 @@ void shock_serpent_discharge_fineff::fire()
         mprf("The air sparks with electricity, shocking %s!",
              oppressor.name(DESC_THE).c_str());
     }
-
+    bolt beam;
+    beam.flavour = BEAM_ELECTRICITY;
 
     int amount = roll_dice(3, 4 + power * 3 / 2);
-    amount = oppressor.apply_ac(amount, 0, AC_HALF);
-    oppressor.hurt(serpent, amount, BEAM_ELECTRICITY,
-                   KILLED_BY_BEAM,
-                   "a shock serpent", "electric aura");
+    amount = oppressor.apply_ac(oppressor.beam_resists(beam, amount, true),
+                                                                0, AC_HALF);
+    oppressor.hurt(serpent, amount, beam.flavour, KILLED_BY_BEAM,
+                                        "a shock serpent", "electric aura");
+    if (amount)
+        oppressor.expose_to_element(beam.flavour, amount);
 }
 
 void delayed_action_fineff::fire()
