@@ -12,6 +12,7 @@
 #include "branch.h"
 #include "dgn-overview.h"
 #include "message.h"
+#include "mon-death.h"
 #include "mon-util.h"
 #include "religion.h"
 #include "spl-other.h"
@@ -265,4 +266,24 @@ void fixup_bad_companions()
             ++i;
     }
 }
+
+bool maybe_bad_priest_monster(monster &mons)
+{
+    // prior to e6d7efa92cb0, if a follower got polymorphed to a form that
+    // satisfied is_priest, its god got cleared. This resulted in Beogh
+    // followers potentially getting cloned on level load, resulting in
+    // duplicate mids or a corrupted mid cache depending on ordering. This is
+    // now fixed up in tag_read_level_load.
+    return mons.alive() && mons.attitude == ATT_FRIENDLY
+                        && mons.god == GOD_NAMELESS;
+}
+
+void fixup_bad_priest_monster(monster &mons)
+{
+    if (!maybe_bad_priest_monster(mons))
+        return;
+    mprf(MSGCH_ERROR, "Removing corrupted ex-follower from level: %s.",
+                                            mons.full_name(DESC_PLAIN).c_str());
+    monster_die(mons, KILL_RESET, -1, true, false);
+}
 #endif
