@@ -1477,6 +1477,7 @@ int msgwin_get_line(string prompt, char *buf, int len,
         reader.read_line(fill);
         reader.putkey(CK_END);
 
+        linebreak_string(prompt, 79);
         msg_colour_type colour = prepare_message(prompt, MSGCH_PROMPT, 0);
         const string colour_prompt = colour_string(prompt, colour_msg(colour));
 
@@ -1972,7 +1973,11 @@ string get_last_messages(int mcount, bool full)
         if (!msg)
             break;
         if (full || is_channel_dumpworthy(msg.channel))
-            text = msg.pure_text_with_repeats() + "\n" + text;
+        {
+            string line = msg.pure_text_with_repeats();
+            string wrapped = wordwrap_line(line, 79, false, true);
+            text = wrapped + "\n" + text;
+        }
         mcount--;
     }
 
