@@ -664,6 +664,7 @@ unsigned int item_value(item_def item, bool ident)
                 case AMU_THE_GOURMAND:
                 case AMU_HARM:
                 case AMU_MANA_REGENERATION:
+                case AMU_ACROBAT:
                     valued += 300;
                     break;
 
@@ -691,7 +692,7 @@ unsigned int item_value(item_def item, bool ident)
                     valued += 150;
                     break;
 
-                case RING_LOUDNESS:
+                case RING_ATTENTION:
                 case RING_TELEPORTATION:
                 case AMU_NOTHING:
                     valued += 75;
@@ -964,7 +965,6 @@ class ShopMenu : public InvMenu
     friend class ShopEntry;
 
     shop_struct& shop;
-    bool looking = false;
     shopping_order order = ORDER_DEFAULT;
     level_pos pos;
     bool can_purchase;
@@ -977,11 +977,6 @@ class ShopMenu : public InvMenu
     void purchase_selected();
 
     virtual bool process_key(int keyin) override;
-    virtual void draw_menu() override;
-    // Selecting one item may remove another from the shopping list, or change
-    // the colour others need to have, so we can't be lazy with redrawing
-    // elements.
-    virtual bool always_redraw() const override { return true; }
 
 public:
     bool bought_something = false;
@@ -1054,8 +1049,8 @@ ShopMenu::ShopMenu(shop_struct& _shop, const level_pos& _pos, bool _can_purchase
       pos(_pos),
       can_purchase(_can_purchase)
 {
-    if (!can_purchase)
-        looking = true;
+    menu_action = can_purchase ? ACT_EXECUTE : ACT_EXAMINE;
+    set_flags(get_flags() & ~MF_USE_TWO_COLUMNS);
 
     set_tag("shop");
 
@@ -1079,25 +1074,6 @@ void ShopMenu::init_entries()
     }
 }
 
-void ShopMenu::draw_menu()
-{
-    // Unlike other menus, selecting one item in the shopping menu may change
-    // other ones (because of the colour scheme). Keypresses also need to
-    // update the more, which is hijacked for use as help text.
-#ifdef USE_TILE_WEB
-    tiles.json_open_object();
-    tiles.json_write_string("msg", "update_menu");
-    tiles.json_write_string("more", more.to_colour_string());
-    tiles.json_write_int("total_items", items.size());
-    tiles.json_close_object();
-    tiles.finish_message();
-    if (items.size() > 0)
-        webtiles_update_items(0, items.size() - 1);
-#endif
-
-    InvMenu::draw_menu();
-}
-
 int ShopMenu::selected_cost() const
 {
     int cost = 0;
@@ -1128,7 +1104,12 @@ void ShopMenu::update_help()
                          you.gold - total_cost,
                          (you.gold - total_cost != 1) ? "s" : "");
     }
-    top_line += "</yellow>\n";
+    top_line += "</yellow>";
+
+    // Ensure length >= 80ch, which prevents the local tiles menu from resizing
+    // as the player selects/deselects entries. Blegh..
+    int top_line_width = strwidth(formatted_string::parse_string(top_line).tostring());
+    top_line += string(max(0, 80 - top_line_width), ' ') + '\n';
 
     set_more(formatted_string::parse_string(top_line + make_stringf(
         //You have 0 gold pieces.
@@ -1143,10 +1124,10 @@ void ShopMenu::update_help()
         "%s  [%s] %s\n"
         "[<w>/</w>] sort (%s)%s  %s  [%s] put item on shopping list",
         !can_purchase ? " " " "  "  " "       "  "          " :
-        looking       ? "[<w>!</w>] buy|<w>examine</w> items" :
-                        "[<w>!</w>] <w>buy</w>|examine items",
+        menu_action == ACT_EXECUTE ? "[<w>!</w>] <w>buy</w>|examine items" :
+                                     "[<w>!</w>] buy|<w>examine</w> items",
         _hyphenated_letters(item_count(), 'a').c_str(),
-        looking ? "examine item" : "select item for purchase",
+        menu_action == ACT_EXECUTE ? "select item for purchase" : "examine item",
         shopping_order_names[order],
         // strwidth("default")
         string(7 - strwidth(shopping_order_names[order]), ' ').c_str(),
@@ -1186,7 +1167,7 @@ void ShopMenu::purchase_selected()
                    col.c_str(),
                    col.c_str()));
         more += old_more;
-        draw_menu();
+        update_more();
         return;
     }
     more = formatted_string::parse_string(make_stringf(
@@ -1196,11 +1177,11 @@ void ShopMenu::purchase_selected()
                cost,
                col.c_str()));
     more += old_more;
-    draw_menu();
+    update_more();
     if (!yesno(nullptr, true, 'n', false, false, true))
     {
         more = old_more;
-        draw_menu();
+        update_more();
         return;
     }
     sort(begin(selected), end(selected),
@@ -1250,6 +1231,8 @@ void ShopMenu::purchase_selected()
 
     if (outside_items)
     {
+        update_help();
+        const formatted_string next_more = more;
         more = formatted_string::parse_string(make_stringf(
             "<%s>I'll put %s outside for you.</%s>\n",
             col.c_str(),
@@ -1257,12 +1240,13 @@ void ShopMenu::purchase_selected()
       (int) bought_indices.size() == outside_items ? "them"
                                                    : "some of them",
             col.c_str()));
-        more += old_more;
+        more += next_more;
+        update_more();
     }
     else
         update_help();
 
-    draw_menu();
+    update_menu(true);
 }
 
 // Doesn't handle redrawing itself.
@@ -1321,9 +1305,12 @@ bool ShopMenu::process_key(int keyin)
     case '?':
         if (can_purchase)
         {
-            looking = !looking;
+            if (menu_action == ACT_EXECUTE)
+                menu_action = ACT_EXAMINE;
+            else
+                menu_action = ACT_EXECUTE;
             update_help();
-            draw_menu();
+            update_more();
         }
         return true;
     case ' ':
@@ -1352,20 +1339,21 @@ bool ShopMenu::process_key(int keyin)
                 if (shopping_list.is_on_list(*dynamic_cast<ShopEntry*>(entry)->item, &pos))
                     entry->select(-2);
         // Move shoplist to selection.
-        draw_menu();
+        update_menu(true);
         return true;
     }
     case '/':
         ++order;
         resort();
         update_help();
-        draw_menu();
+        update_menu(true);
         return true;
     default:
         break;
     }
 
-    if (keyin - 'a' >= 0 && keyin - 'a' < (int)items.size() && looking)
+    if (keyin - 'a' >= 0 && keyin - 'a' < (int)items.size()
+        && menu_action == ACT_EXAMINE)
     {
         item_def& item(*const_cast<item_def*>(dynamic_cast<ShopEntry*>(
             items[letter_to_index(keyin)])->item));
@@ -1385,7 +1373,6 @@ bool ShopMenu::process_key(int keyin)
             }
             describe_item(item);
         }
-        draw_menu();
         return true;
     }
     else if (keyin - 'A' >= 0 && keyin - 'A' < (int)items.size())
@@ -1398,8 +1385,7 @@ bool ShopMenu::process_key(int keyin)
             shopping_list.del_thing(item, &pos);
         else
             shopping_list.add_thing(item, item_price(item, shop), &pos);
-        // not draw_item since other items may enter/leave shopping list
-        draw_menu();
+        update_menu(true);
         return true;
     }
 
@@ -1409,7 +1395,7 @@ bool ShopMenu::process_key(int keyin)
     {
         // Update the footer to display the new $$$ info.
         update_help();
-        draw_menu();
+        update_menu(true);
     }
     return ret;
 }
@@ -1671,11 +1657,6 @@ ShoppingList::ShoppingList()
         pos = level_pos::current(); \
     ASSERT(pos.is_valid());
 
-#define SETUP_THING()                             \
-    CrawlHashTable *thing = new CrawlHashTable();  \
-    (*thing)[SHOPPING_THING_COST_KEY] = cost; \
-    (*thing)[SHOPPING_THING_POS_KEY]  = pos;
-
 bool ShoppingList::add_thing(const item_def &item, int cost,
                              const level_pos* _pos)
 {
@@ -1684,14 +1665,16 @@ bool ShoppingList::add_thing(const item_def &item, int cost,
 
     SETUP_POS();
 
-    if (find_thing(item, pos) != -1)
+    if (!find_thing(item, pos).empty()) // TODO: this check isn't working?
     {
         mprf(MSGCH_ERROR, "%s is already on the shopping list.",
              item.name(DESC_THE).c_str());
         return false;
     }
 
-    SETUP_THING();
+    CrawlHashTable *thing = new CrawlHashTable();
+    (*thing)[SHOPPING_THING_COST_KEY] = cost;
+    (*thing)[SHOPPING_THING_POS_KEY]  = pos;
     (*thing)[SHOPPING_THING_ITEM_KEY] = item;
     list->push_back(*thing);
     refresh();
@@ -1699,45 +1682,18 @@ bool ShoppingList::add_thing(const item_def &item, int cost,
     return true;
 }
 
-bool ShoppingList::add_thing(string desc, string buy_verb, int cost,
-                             const level_pos* _pos)
-{
-    ASSERT(!desc.empty());
-    ASSERT(!buy_verb.empty());
-    ASSERT(cost > 0);
-
-    SETUP_POS();
-
-    if (find_thing(desc, pos) != -1)
-    {
-        mprf(MSGCH_ERROR, "%s is already on the shopping list.",
-             desc.c_str());
-        return false;
-    }
-
-    SETUP_THING();
-    (*thing)[SHOPPING_THING_DESC_KEY] = desc;
-    (*thing)[SHOPPING_THING_VERB_KEY] = buy_verb;
-    list->push_back(*thing);
-    refresh();
-
-    return true;
-}
-
-#undef SETUP_THING
-
 bool ShoppingList::is_on_list(const item_def &item, const level_pos* _pos) const
 {
     SETUP_POS();
 
-    return find_thing(item, pos) != -1;
+    return !find_thing(item, pos).empty();
 }
 
 bool ShoppingList::is_on_list(string desc, const level_pos* _pos) const
 {
     SETUP_POS();
 
-    return find_thing(desc, pos) != -1;
+    return !find_thing(desc, pos).empty();
 }
 
 void ShoppingList::del_thing_at_index(int idx)
@@ -1747,6 +1703,19 @@ void ShoppingList::del_thing_at_index(int idx)
     refresh();
 }
 
+template <typename C>
+void ShoppingList::del_thing_at_indices(C const &idxs)
+{
+    set<int,greater<int>> indices(idxs.begin(), idxs.end());
+
+    for (auto idx : indices)
+    {
+        ASSERT_RANGE(idx, 0, list->size());
+        list->erase(idx);
+    }
+    refresh();
+}
+
 void ShoppingList::del_things_from(const level_id &lid)
 {
     for (unsigned int i = 0; i < list->size(); i++)
@@ -1764,16 +1733,16 @@ bool ShoppingList::del_thing(const item_def &item,
 {
     SETUP_POS();
 
-    int idx = find_thing(item, pos);
+    auto indices = find_thing(item, pos);
 
-    if (idx == -1)
+    if (indices.empty())
     {
         mprf(MSGCH_ERROR, "%s isn't on shopping list, can't delete it.",
              item.name(DESC_THE).c_str());
         return false;
     }
 
-    del_thing_at_index(idx);
+    del_thing_at_indices(indices);
     return true;
 }
 
@@ -1781,16 +1750,16 @@ bool ShoppingList::del_thing(string desc, const level_pos* _pos)
 {
     SETUP_POS();
 
-    int idx = find_thing(desc, pos);
+    auto indices = find_thing(desc, pos);
 
-    if (idx == -1)
+    if (indices.empty())
     {
         mprf(MSGCH_ERROR, "%s isn't on shopping list, can't delete it.",
              desc.c_str());
         return false;
     }
 
-    del_thing_at_index(idx);
+    del_thing_at_indices(indices);
     return true;
 }
 
@@ -1853,7 +1822,7 @@ bool ShoppingList::cull_identical_items(const item_def& item, int cost)
         return 0;
 
     // Item is already on shopping-list.
-    const bool on_list = find_thing(item, level_pos::current()) != -1;
+    const bool on_list = !find_thing(item, level_pos::current()).empty();
 
     const bool do_prompt = item.base_type == OBJ_JEWELLERY
                            && !jewellery_is_amulet(item)
@@ -2018,6 +1987,42 @@ void ShoppingList::item_type_identified(object_class_type base_type,
     refresh();
 }
 
+void ShoppingList::spells_added_to_library(const vector<spell_type>& spells, bool quiet)
+{
+    if (!list) /* let's not make book starts crash instantly... */
+        return;
+
+    unordered_set<int> indices_to_del;
+    for (CrawlHashTable &thing : *list)
+    {
+        if (!thing_is_item(thing)) // ???
+            continue;
+        const item_def& book = get_thing_item(thing);
+        if (book.base_type != OBJ_BOOKS || book.sub_type == BOOK_MANUAL)
+            continue;
+
+        const auto item_spells = spells_in_book(book);
+        if (any_of(item_spells.begin(), item_spells.end(), [&spells](const spell_type st) {
+                    return find(spells.begin(), spells.end(), st) != spells.end();
+                }) && is_useless_item(book, false))
+        {
+            level_pos pos = thing_pos(thing); // TODO: unreliable?
+            auto thing_indices = find_thing(get_thing_item(thing), pos);
+            indices_to_del.insert(thing_indices.begin(), thing_indices.end());
+        }
+    }
+    if (!quiet)
+    {
+        for (auto idx : indices_to_del)
+        {
+            ASSERT_RANGE(idx, 0, list->size());
+            mprf("Shopping list: removing %s",
+                describe_thing((*list)[idx], DESC_A).c_str());
+        }
+    }
+    del_thing_at_indices(indices_to_del);
+}
+
 void ShoppingList::remove_dead_shops()
 {
     // Only restore the excursion at the very end.
@@ -2158,54 +2163,48 @@ void ShoppingList::gold_changed(int old_amount, int new_amount)
 class ShoppingListMenu : public Menu
 {
 public:
-    ShoppingListMenu()
-#ifdef USE_TILE_LOCAL
-        : Menu(MF_MULTISELECT,"",false)
-#else
-        : Menu()
-#endif
-    { }
+    ShoppingListMenu() : Menu(MF_MULTISELECT | MF_ALLOW_FORMATTING) {}
+    bool view_only {false};
 
 protected:
-    void draw_title();
+    virtual formatted_string calc_title() override;
 };
 
-void ShoppingListMenu::draw_title()
+formatted_string ShoppingListMenu::calc_title()
 {
-    if (title)
+    formatted_string fs;
+    const int total_cost = you.props[SHOPPING_LIST_COST_KEY];
+
+    fs.textcolour(title->colour);
+    fs.cprintf("%d %s%s, total %d gold",
+                title->quantity, title->text.c_str(),
+                title->quantity > 1 ? "s" : "",
+                total_cost);
+
+    string s = "<lightgrey>  [<w>a-z</w>] ";
+
+    if (view_only)
     {
-        const int total_cost = you.props[SHOPPING_LIST_COST_KEY];
-
-        cgotoxy(1, 1);
-        formatted_string fs = formatted_string(title->colour);
-        fs.cprintf("%d %s%s, total %d gold",
-                   title->quantity, title->text.c_str(),
-                   title->quantity > 1? "s" : "",
-                   total_cost);
-        fs.display();
-
-#ifdef USE_TILE_WEB
-        webtiles_set_title(fs);
-#endif
-        string s = "<lightgrey>  [<w>a-z</w>] ";
-
-        switch (menu_action)
-        {
-        case ACT_EXECUTE:
-            s += "<w>travel</w>|examine|delete";
-            break;
-        case ACT_EXAMINE:
-            s += "travel|<w>examine</w>|delete";
-            break;
-        default:
-            s += "travel|examine|<w>delete</w>";
-            break;
-        }
-
-        s += "  [<w>?</w>/<w>!</w>] change action</lightgrey>";
-
-        draw_title_suffix(formatted_string::parse_string(s), false);
+        fs += formatted_string::parse_string(s + "<w>examine</w>");
+        return fs;
     }
+
+    switch (menu_action)
+    {
+    case ACT_EXECUTE:
+        s += "<w>travel</w>|examine|delete";
+        break;
+    case ACT_EXAMINE:
+        s += "travel|<w>examine</w>|delete";
+        break;
+    default:
+        s += "travel|examine|<w>delete</w>";
+        break;
+    }
+
+    s += "  [<w>?</w>/<w>!</w>] change action</lightgrey>";
+    fs += formatted_string::parse_string(s);
+    return fs;
 }
 
 /**
@@ -2245,9 +2244,7 @@ void ShoppingList::fill_out_menu(Menu& shopmenu)
         MenuEntry *me = new MenuEntry(etitle, MEL_ITEM, 1, hotkey);
         me->data = &thing;
 
-        if (cost > you.gold)
-            me->colour = DARKGREY;
-        else if (thing_is_item(thing))
+        if (thing_is_item(thing))
         {
             // Colour shopping list item according to menu colours.
             const item_def &item = get_thing_item(thing);
@@ -2256,36 +2253,40 @@ void ShoppingList::fill_out_menu(Menu& shopmenu)
             const int col = menu_colour(item.name(DESC_A),
                                         colprf, "shop");
 
+#ifdef USE_TILE
+            vector<tile_def> item_tiles;
+            get_tiles_for_item(item, item_tiles, true);
+            for (const auto &tile : item_tiles)
+                me->add_tile(tile);
+#endif
+
             if (col != -1)
                 me->colour = col;
         }
+        if (cost > you.gold)
+            me->colour = DARKGREY;
 
         shopmenu.add_entry(me);
         ++hotkey;
     }
 }
 
-void ShoppingList::display()
+void ShoppingList::display(bool view_only)
 {
     if (list->empty())
         return;
 
     ShoppingListMenu shopmenu;
+    shopmenu.view_only = view_only;
     shopmenu.set_tag("shop");
-    shopmenu.menu_action  = Menu::ACT_EXECUTE;
-    shopmenu.action_cycle = Menu::CYCLE_CYCLE;
-    string title          = "thing";
+    shopmenu.menu_action  = view_only ? Menu::ACT_EXAMINE : Menu::ACT_EXECUTE;
+    shopmenu.action_cycle = view_only ? Menu::CYCLE_NONE : Menu::CYCLE_CYCLE;
+    string title          = "item";
 
     MenuEntry *mtitle = new MenuEntry(title, MEL_TITLE);
+    mtitle->quantity = list->size();
     shopmenu.set_title(mtitle);
 
-    // Don't make a menu so tall that we recycle hotkeys on the same page.
-    if (list->size() > 52
-        && (shopmenu.maxpagesize() > 52 || shopmenu.maxpagesize() == 0))
-    {
-        shopmenu.set_maxpagesize(52);
-    }
-
     string more_str = make_stringf("<yellow>You have %d gp</yellow>", you.gold);
     shopmenu.set_more(formatted_string::parse_string(more_str));
 
@@ -2294,20 +2295,11 @@ void ShoppingList::display()
 
     fill_out_menu(shopmenu);
 
-    vector<MenuEntry*> sel;
-    while (true)
+    shopmenu.on_single_selection =
+        [this, &shopmenu, &mtitle](const MenuEntry& sel)
     {
-        // Abuse of the quantity field.
-        mtitle->quantity = list->size();
-
-        redraw_screen();
-        sel = shopmenu.show();
-
-        if (sel.empty())
-            break;
-
         const CrawlHashTable* thing =
-            static_cast<const CrawlHashTable *>(sel[0]->data);
+            static_cast<const CrawlHashTable *>(sel.data);
 
         const bool is_item = thing_is_item(*thing);
 
@@ -2323,16 +2315,15 @@ void ShoppingList::display()
                                 describe_thing(*thing, DESC_A).c_str());
                 clrscr();
                 if (!yesno(prompt.c_str(), true, 'n'))
-                    continue;
+                    return true;
             }
 
             const level_pos lp(thing_pos(*thing));
             start_translevel_travel(lp);
-            break;
+            return false;
         }
         else if (shopmenu.menu_action == Menu::ACT_EXAMINE)
         {
-            clrscr();
             if (is_item)
             {
                 const item_def &item = get_thing_item(*thing);
@@ -2345,34 +2336,39 @@ void ShoppingList::display()
                              "%s with an entry fee of %d gold pieces.",
                              describe_thing(*thing, DESC_A).c_str(),
                              (int) thing_cost(*thing));
-
-                print_description(info.c_str());
-                getchm();
+                show_description(info.c_str());
             }
         }
         else if (shopmenu.menu_action == Menu::ACT_MISC)
         {
-            const int index = shopmenu.get_entry_index(sel[0]);
+            const int index = shopmenu.get_entry_index(&sel);
             if (index == -1)
             {
                 mprf(MSGCH_ERROR, "ERROR: Unable to delete thing from shopping list!");
                 more();
-                continue;
+                return true;
             }
 
             del_thing_at_index(index);
+            mtitle->quantity = list->size();
+            shopmenu.set_title(mtitle);
+
             if (list->empty())
             {
                 mpr("Your shopping list is now empty.");
-                break;
+                return false;
             }
 
             shopmenu.clear();
             fill_out_menu(shopmenu);
+            shopmenu.update_menu(true);
         }
         else
             die("Invalid menu action type");
-    }
+        return true;
+    };
+
+    shopmenu.show();
     redraw_screen();
 }
 
@@ -2427,15 +2423,16 @@ void ShoppingList::refresh()
     you.props[SHOPPING_LIST_COST_KEY].get_int() = total_cost;
 }
 
-int ShoppingList::find_thing(const item_def &item,
+unordered_set<int> ShoppingList::find_thing(const item_def &item,
                              const level_pos &pos) const
 {
+    unordered_set<int> result;
     for (unsigned int i = 0; i < list->size(); i++)
     {
         const CrawlHashTable &thing = (*list)[i];
         const level_pos       _pos  = thing_pos(thing);
 
-        if (pos != _pos)
+        if (pos != _pos) // TODO: using thing_pos above seems to make this unreliable?
             continue;
 
         if (!thing_is_item(thing))
@@ -2444,14 +2441,15 @@ int ShoppingList::find_thing(const item_def &item,
         const item_def &_item = get_thing_item(thing);
 
         if (item_name_simple(item) == item_name_simple(_item))
-            return i;
+            result.insert(i);
     }
 
-    return -1;
+    return result;
 }
 
-int ShoppingList::find_thing(const string &desc, const level_pos &pos) const
+unordered_set<int> ShoppingList::find_thing(const string &desc, const level_pos &pos) const
 {
+    unordered_set<int> result;
     for (unsigned int i = 0; i < list->size(); i++)
     {
         const CrawlHashTable &thing = (*list)[i];
@@ -2464,10 +2462,10 @@ int ShoppingList::find_thing(const string &desc, const level_pos &pos) const
             continue;
 
         if (desc == name_thing(thing))
-            return i;
+            result.insert(i);
     }
 
-    return -1;
+    return result;
 }
 
 bool ShoppingList::thing_is_item(const CrawlHashTable& thing)
@@ -2523,10 +2521,9 @@ string ShoppingList::name_thing(const CrawlHashTable& thing,
 string ShoppingList::describe_thing(const CrawlHashTable& thing,
                                     description_level_type descrip)
 {
-    const level_pos pos = thing_pos(thing);
-
     string desc = name_thing(thing, descrip) + " on ";
 
+    const level_pos pos = thing_pos(thing);
     if (pos.id == level_id::current())
         desc += "this level";
     else
