@@ -10,6 +10,7 @@
 #include "cloud.h"
 #include "command.h"
 #include "coord.h"
+#include "describe.h"
 #include "directn.h"
 #include "dgn-height.h"
 #include "env.h"
@@ -24,13 +25,13 @@
 #include "nearby-danger.h"
 #include "options.h"
 #include "output.h"
-#include "process-desc.h"
 #include "prompt.h"
 #include "religion.h"
 #include "spl-book.h"
 #include "spl-cast.h"
 #include "spl-zap.h"
 #include "stash.h"
+#include "stringutil.h"
 #include "terrain.h"
 #include "tiledef-dngn.h"
 #include "tiledef-icons.h"
@@ -374,7 +375,7 @@ static void _add_targeting_commands(const coord_def& pos)
 {
     // Force targeting cursor back onto center to start off on a clean
     // slate.
-    macro_buf_add_cmd(CMD_TARGET_FIND_YOU);
+    macro_sendkeys_end_add_cmd(CMD_TARGET_FIND_YOU);
 
     const coord_def delta = pos - you.pos();
 
@@ -386,7 +387,7 @@ static void _add_targeting_commands(const coord_def& pos)
         cmd = CMD_TARGET_RIGHT;
 
     for (int i = 0; i < abs(delta.x); i++)
-        macro_buf_add_cmd(cmd);
+        macro_sendkeys_end_add_cmd(cmd);
 
     if (delta.y < 0)
         cmd = CMD_TARGET_UP;
@@ -394,9 +395,9 @@ static void _add_targeting_commands(const coord_def& pos)
         cmd = CMD_TARGET_DOWN;
 
     for (int i = 0; i < abs(delta.y); i++)
-        macro_buf_add_cmd(cmd);
+        macro_sendkeys_end_add_cmd(cmd);
 
-    macro_buf_add_cmd(CMD_TARGET_MOUSE_SELECT);
+    macro_sendkeys_end_add_cmd(CMD_TARGET_MOUSE_SELECT);
 }
 
 static bool _is_appropriate_spell(spell_type spell, const actor* target)
@@ -516,7 +517,7 @@ static bool _evoke_item_on_target(actor* target)
     }
 #endif
 
-    macro_buf_add_cmd(CMD_EVOKE);
+    macro_sendkeys_end_add_cmd(CMD_EVOKE);
     macro_buf_add(index_to_letter(item->link)); // Inventory letter.
     _add_targeting_commands(target->pos());
     return true;
@@ -599,7 +600,7 @@ static bool _cast_spell_on_target(actor* target)
         return true;
     }
 
-    macro_buf_add_cmd(CMD_FORCE_CAST_SPELL);
+    macro_sendkeys_end_add_cmd(CMD_FORCE_CAST_SPELL);
     macro_buf_add(letter);
 
     if (get_spell_flags(spell) & SPFLAG_TARGETING_MASK)
@@ -643,7 +644,7 @@ static bool _handle_distant_monster(monster* mon, unsigned char mod)
         && (shift || weapon && is_range_weapon(*weapon)
                      && !mon->wont_attack()))
     {
-        macro_buf_add_cmd(CMD_FIRE);
+        macro_sendkeys_end_add_cmd(CMD_FIRE);
         _add_targeting_commands(mon->pos());
         return true;
     }
@@ -659,7 +660,7 @@ static bool _handle_distant_monster(monster* mon, unsigned char mod)
 
         if (dist > 1 && weapon && weapon_reach(*weapon) >= dist)
         {
-            macro_buf_add_cmd(CMD_EVOKE_WIELDED);
+            macro_sendkeys_end_add_cmd(CMD_EVOKE_WIELDED);
             _add_targeting_commands(mon->pos());
             return true;
         }
@@ -727,12 +728,8 @@ int DungeonRegion::handle_mouse(MouseEvent &event)
         return 0;
 
 #ifdef TOUCH_UI
-    if (event.event == MouseEvent::PRESS
-        && (event.mod & TILES_MOD_CTRL)
-        && (event.button == MouseEvent::SCROLL_UP || event.button == MouseEvent::SCROLL_DOWN))
-    {
+    if (event.event == MouseEvent::WHEEL && (event.mod & TILES_MOD_CTRL))
         zoom(event.button == MouseEvent::SCROLL_UP);
-    }
 #endif
 
     if (mouse_control::current_mode() == MOUSE_MODE_NORMAL
@@ -749,8 +746,7 @@ int DungeonRegion::handle_mouse(MouseEvent &event)
         return CK_MOUSE_CLICK;
     }
 
-    if (mouse_control::current_mode() == MOUSE_MODE_NORMAL
-        || mouse_control::current_mode() == MOUSE_MODE_MACRO
+    if (mouse_control::current_mode() == MOUSE_MODE_MACRO
         || mouse_control::current_mode() == MOUSE_MODE_MORE
         || mouse_control::current_mode() == MOUSE_MODE_PROMPT
         || mouse_control::current_mode() == MOUSE_MODE_YESNO)
@@ -789,6 +785,9 @@ int DungeonRegion::handle_mouse(MouseEvent &event)
             tiles.add_text_tag(TAG_CELL_DESC, desc, gc);
     }
 
+    if (mouse_control::current_mode() == MOUSE_MODE_NORMAL)
+        return 0;
+
     if (!on_map)
         return 0;
 
@@ -1015,7 +1014,7 @@ bool DungeonRegion::update_tip_text(string &tip)
             tip += make_stringf("GC(%d, %d) EP(%d, %d)\n",
                                 gc.x, gc.y, ep.x, ep.y);
 
-            if (env.heightmap.get())
+            if (env.heightmap)
                 tip += make_stringf("HEIGHT(%d)\n", dgn_height_at(gc));
 
             tip += "\n";
@@ -1024,7 +1023,7 @@ bool DungeonRegion::update_tip_text(string &tip)
         else
         {
             tip += make_stringf("GC(%d, %d) [out of sight]\n", gc.x, gc.y);
-            if (env.heightmap.get())
+            if (env.heightmap)
                 tip += make_stringf("HEIGHT(%d)\n", dgn_height_at(gc));
             tip += "\n";
         }
@@ -1288,10 +1287,7 @@ bool DungeonRegion::update_alt_text(string &alt)
             inf.body << "\n" << stash;
     }
 
-    alt_desc_proc proc(crawl_view.msgsz.x, crawl_view.msgsz.y);
-    process_description<alt_desc_proc>(proc, inf);
-
-    proc.get_string(alt);
+    alt = process_description(inf);
 
     // Suppress floor description
     if (alt == "Floor.")
