@@ -3,10 +3,10 @@
  * @brief Traps related functions.
 **/
 
-#ifndef TRAPS_H
-#define TRAPS_H
+#pragma once
 
 #include "enum.h"
+#include "trap-type.h"
 
 #define NEWLY_TRAPPED_KEY "newly_trapped"
 
@@ -47,4 +47,6 @@ trap_type random_vault_trap();
 int count_traps(trap_type ttyp);
 void place_webs(int num);
 bool ensnare(actor *fly);
-#endif
+void leave_web(bool quiet = false);
+void monster_web_cleanup(const monster &mons, bool quiet = false);
+void stop_being_held();
