@@ -193,7 +193,7 @@ static void _do_wizard_command(int wiz_command)
     case '(': wizard_create_feature(); break;
     // case ')': break;
 
-    // case '`': break;
+    case '`': debug_list_vacant_keys(); break;
     case '~': wizard_interlevel_travel(); break;
 
     case '-': wizard_get_god_gift(); break;
@@ -406,9 +406,6 @@ int list_wizard_commands(bool do_redraw_screen)
 {
     // 2 columns
     column_composer cols(2, 44);
-    // Page size is number of lines - one line for --more-- prompt.
-    cols.set_pagesize(get_number_of_lines());
-
     cols.add_formatted(0,
                        "<yellow>Player stats</yellow>\n"
                        "<w>A</w>      set all skills to level\n"
@@ -510,6 +507,7 @@ int list_wizard_commands(bool do_redraw_screen)
 #endif
                        "<w>Ctrl-Y</w> temporarily suppress wizmode\n"
                        "<w>Ctrl-C</w> force a crash\n"
+                       "<w>`</w>      list unassigned command keys\n"
                        "\n"
                        "<yellow>Other wizard commands</yellow>\n"
                        "(not prefixed with <w>&</w>!)\n"
@@ -517,8 +515,7 @@ int list_wizard_commands(bool do_redraw_screen)
                        "<w>X?</w>     list map-mode commands\n",
                        true);
 
-    int key = show_keyhelp_menu(cols.formatted_lines(), false,
-                                Options.easy_exit_menu);
+    int key = show_keyhelp_menu(cols.formatted_lines());
     if (do_redraw_screen)
         redraw_screen();
     return key;
